MODULE defs
  REAL(8),PARAMETER       :: PI =     3.1415926535897932d0
  REAL(8),PARAMETER       :: ZERO =   0.0d0
  REAL(8),PARAMETER       :: HALF =   0.5d0
  REAL(8),PARAMETER       :: ONE =    1.0d0
  REAL(8),PARAMETER       :: TWO =    2.0d0
  REAL(8),PARAMETER       :: THREE =  3.0d0
  REAL(8),PARAMETER       :: FOUR  =  4.0d0
  REAL(8),PARAMETER       :: NINETY = 90.0d0
  COMPLEX(8),PARAMETER    :: CZERO =  (0.0d0,0.0d0)
  COMPLEX(8),PARAMETER    :: IMAG =   (0.0D0,1.0D0)
  COMPLEX(8),PARAMETER    :: CONE =   (1.0D0,0.0D0)
  REAL(8),PARAMETER       :: TEST =   1.D-12
  INTEGER,PARAMETER       :: IUNITY(3,3) = RESHAPE((/1,0,0,0,1,0,0,0,1/),(/3,3/))
  REAL(8),PARAMETER       :: UNITY(3,3) = RESHAPE((/ONE,ZERO,ZERO,ZERO,ONE,ZERO,ZERO,ZERO,ONE/),(/3,3/))
END MODULE defs

MODULE constants
  USE defs
  !** Update: 5/1-05 from physics.nist.gov/constants                    **
  !**  atomic units:    HBAR=ECHARGE=EMASS=4*PI*EPSILON_0=1             **
  REAL(8),PARAMETER   :: HBAR        = ONE
  REAL(8),PARAMETER   :: ECHARGE     = ONE
  REAL(8),PARAMETER   :: EMASS       = ONE
  REAL(8),PARAMETER   :: EPSILON0    = ONE/(FOUR*PI)
  REAL(8),PARAMETER   :: A0          = ONE

  REAL(8),PARAMETER   :: FINE_STRUCT = 7.297352568D-3
  REAL(8),PARAMETER   :: Clight      = ONE/FINE_STRUCT

  REAL(8),PARAMETER   :: MOL         = 6.0221415D23
  REAL(8),PARAMETER   :: BOLTZMANN   = 3.166815D-6     ! Ha/K
  !**  DERIVED ATOMIC UNITS 
  REAL(8),PARAMETER   :: MU0          = 1.D0/(EPSILON0*Clight**2)   ! PERMEABILITY OF VACUUM

  ! Constants defining SI units
  REAL(8),PARAMETER   :: Clight_SI    = 299792458D0      ! [M/S]
  REAL(8),PARAMETER   :: Mu0_SI       = FOUR*PI*1.D-7      ! [N/A^2]
  REAL(8),PARAMETER   :: Hbar_SI      = 1.05457168D-34   ! PLANCK CONSTANT [Js]
  REAL(8),PARAMETER   :: Echarge_SI   = 1.602176462D-19  ! ELECTRON CHARGE [C]
  REAL(8),PARAMETER   :: Emass_SI     = 9.1093826D-31    ! ELECTRON MASS   [kg]
  REAL(8),PARAMETER   :: Epsilon0_SI  = ONE/(MU0_SI*Clight_SI**2) !        [C/Vm]
  REAL(8),PARAMETER   :: A0_SI        = 0.5291772083D-10 ! Bohr radius [m]
  
  !** fundamental SI units in atomic units    
  REAL(8),PARAMETER   :: METER = A0 / A0_SI
  REAL(8),PARAMETER   :: KG    =EMASS/EMASS_SI 
  REAL(8),PARAMETER   :: SECOND=Clight_SI/Clight*METER
  REAL(8),PARAMETER   :: AMPERE=ECHARGE/(ECHARGE_SI*SECOND)
  !  derived SI units        
  REAL(8),PARAMETER   :: NEWTON=KG*METER/SECOND**2 
  REAL(8),PARAMETER   :: JOULE=NEWTON*METER       
  REAL(8),PARAMETER   :: COULOMB=AMPERE*SECOND 
  REAL(8),PARAMETER   :: TESLA=KG/(COULOMB*SECOND)
  REAL(8),PARAMETER   :: VOLT=JOULE/COULOMB
  REAL(8),PARAMETER   :: PASCAL=JOULE/METER**3
  REAL(8),PARAMETER   :: GAUSS=1.D-4*TESLA
  REAL(8),PARAMETER   :: BAR=1.D+5*PASCAL
  REAL(8),PARAMETER   :: HERTZ=1.D0/SECOND
  REAL(8),PARAMETER   :: OHM=VOLT*METER/AMPERE

  REAL(8),PARAMETER   :: AMU_SI        = ONE/(MOL*1000.d0)
  REAL(8),PARAMETER   :: AMU           = AMU_SI*KG
  REAL(8),PARAMETER   :: Nmass_AMU     = 1.00866491560   ! Neutron mass
  REAL(8),PARAMETER   :: Pmass_AMU     = 1.00727646688   ! Proton mass

! Conversions
  REAL(8),PARAMETER       :: eVprRy      = 13.6058d0
  REAL(8),PARAMETER       :: JpreV       = ECHARGE
END MODULE constants

SUBROUTINE invert_3x3(mat1,inv_mat1,idet)
  USE defs
  IMPLICIT NONE
  REAL(8)        :: mat1(3,3),inv_mat1(3,3),det,determ
  INTEGER        :: idet
  inv_mat1(1,1)=mat1(2,2)*mat1(3,3)-mat1(3,2)*mat1(2,3)                 
  inv_mat1(1,2)=mat1(3,2)*mat1(1,3)-mat1(1,2)*mat1(3,3)                 
  inv_mat1(1,3)=mat1(1,2)*mat1(2,3)-mat1(2,2)*mat1(1,3)                 
  inv_mat1(2,1)=mat1(2,3)*mat1(3,1)-mat1(3,3)*mat1(2,1)                 
  inv_mat1(2,2)=mat1(3,3)*mat1(1,1)-mat1(1,3)*mat1(3,1)                 
  inv_mat1(2,3)=mat1(1,3)*mat1(2,1)-mat1(2,3)*mat1(1,1)                 
  inv_mat1(3,1)=mat1(2,1)*mat1(3,2)-mat1(3,1)*mat1(2,2)                 
  inv_mat1(3,2)=mat1(3,1)*mat1(1,2)-mat1(1,1)*mat1(3,2)                 
  inv_mat1(3,3)=mat1(1,1)*mat1(2,2)-mat1(2,1)*mat1(1,2)                 
  det=determ(mat1)
  idet=1
  IF(det < test) idet=0
  inv_mat1(1:3,1:3)=inv_mat1(1:3,1:3)/det
END SUBROUTINE invert_3x3

REAL(8) FUNCTION determ(a)
  ! returnerer determinaten af a 3x3 matrix
  IMPLICIT NONE
  REAL(8)  :: a(3,3)
  determ=a(1,1)*a(2,2)*a(3,3)+a(1,2)*a(2,3)*a(3,1)+a(1,3)*a(3,2)* &
       a(2,1)-a(1,3)*a(2,2)*a(3,1)-a(2,3)*a(3,2)*a(1,1)-a(3,3)* &
       a(1,2)*a(2,1)
END FUNCTION determ

SUBROUTINE cross(a,b,c)
  REAL(8),INTENT(IN)  :: a(3),b(3)
  REAL(8),INTENT(OUT) :: c(3)
  c(1)=a(2)*b(3)-a(3)*b(2)
  c(2)=a(3)*b(1)-a(1)*b(3)
  c(3)=a(1)*b(2)-a(2)*b(1)
END SUBROUTINE cross
